@@ -19,6 +19,7 @@ import os
 import hashlib
 import binascii
 import logging
+import math
 from urllib.parse import urlparse
 from io import BytesIO
 import shutil
@@ -37,6 +38,12 @@ from url_image_preview.http_functions import get_http_head, get_http_file
 from url_image_preview.config_dialog import UrlImagePreviewConfigDialog
 from url_image_preview.resize_gif import resize_gif
 
+try:
+    from gajim.filechoosers import FileSaveDialog
+    NEW_FILECHOOSER = True
+except ImportError:
+    NEW_FILECHOOSER = False
+
 log = logging.getLogger('gajim.plugin_system.preview')
 
 PILLOW_AVAILABLE = True
@@ -80,7 +87,8 @@ class UrlImagePreviewPlugin(GajimPlugin):
             'print_real_text': (self.print_real_text, None), }
         self.config_default_values = {
             'PREVIEW_SIZE': (150, 'Preview size(10-512)'),
-            'MAX_FILE_SIZE': (524288, 'Max file size for image preview'),
+            'MAX_FILE_SIZE': (5242880, 'Max file size for image preview'),
+            'ALLOW_ALL_IMAGES': (False, ''),
             'LEFTCLICK_ACTION': ('open_menuitem', 'Open'),
             'ANONYMOUS_MUC': (False, ''),
             'GEO_PREVIEW_PROVIDER': ('Google', 'Google Maps'),
@@ -106,9 +114,7 @@ class UrlImagePreviewPlugin(GajimPlugin):
     @log_calls('UrlImagePreviewPlugin')
     def connect_with_history(self, history_window):
         if self.history_window_control:
-            log.error("connect_with_history: deinit handlers")
             self.history_window_control.deinit_handlers()
-        log.error("connect_with_history: create base")
         self.history_window_control = Base(
             self, history_window.history_textview)
 
@@ -139,9 +145,9 @@ class Base(object):
         self.textview = textview
         self.handlers = {}
 
-        self.directory = os.path.join(configpaths.gajimpaths['MY_DATA'],
+        self.directory = os.path.join(configpaths.get('MY_DATA'),
                                       'downloads')
-        self.thumbpath = os.path.join(configpaths.gajimpaths['MY_CACHE'],
+        self.thumbpath = os.path.join(configpaths.get('MY_CACHE'),
                                       'downloads.thumb')
 
         try:
@@ -161,29 +167,10 @@ class Base(object):
 
     def print_real_text(self, real_text, text_tags, graphics, iter_,
                         additional_data):
+
         urlparts = urlparse(real_text)
-        if urlparts.scheme == "geo":
-            if self.plugin.config['GEO_PREVIEW_PROVIDER'] == 'no_preview':
-                return
-        elif (urlparts.scheme not in ["https", "aesgcm"] or not urlparts.netloc):
-            if urlparts.scheme != "geo":
-                log.info("Not accepting URL scheme '%s' for image preview: %s",
-                         urlparts.scheme, real_text)
-                return
-
-        try:
-            oob_url = additional_data["gajim"]["oob_url"]
-        except (KeyError, AttributeError):
-            oob_url = None
-
-        # allow aesgcm uris without oob marker (aesgcm uris are always
-        # httpupload filetransfers)
-        if urlparts.scheme != "aesgcm" and real_text != oob_url:
-            if urlparts.scheme != "geo":
-                log.info("Not accepting URL for image preview "
-                         "(wrong or no oob data): %s", real_text)
-                log.debug("additional_data: %s", additional_data)
-                return
+        if not self._accept_uri(urlparts, real_text, additional_data):
+            return
 
         # Don't print the URL in the message window (in the calling function)
         self.textview.plugin_modified = True
@@ -199,35 +186,6 @@ class Base(object):
             *[(ttt.lookup(t) if isinstance(t, str) else t) for t in ["url"]])
         repl_end = buffer_.create_mark(None, iter_, True)
 
-        if not real_text.startswith('geo:'):
-            weburl = real_text
-            filename = os.path.basename(urlparts.path)
-            ext = os.path.splitext(filename)[1]
-            name = os.path.splitext(filename)[0]
-            namehash = hashlib.sha1(real_text.encode('utf-8')).hexdigest()
-            newfilename = name + '_' + namehash + ext
-            thumbfilename = name + '_' + namehash + '_thumb_' \
-                + str(self.plugin.config['PREVIEW_SIZE']) + ext
-
-            filepath = os.path.join(self.directory, newfilename)
-            thumbpath = os.path.join(self.thumbpath, thumbfilename)
-            filepaths = [filepath, thumbpath]
-
-            key = ''
-            iv = ''
-            encrypted = False
-            if urlparts.fragment:
-                fragment = binascii.unhexlify(urlparts.fragment)
-                key = fragment[16:]
-                iv = fragment[:16]
-                if len(key) == 32 and len(iv) == 16:
-                    encrypted = True
-                if not encrypted:
-                    key = fragment[12:]
-                    iv = fragment[:12]
-                    if len(key) == 32 and len(iv) == 12:
-                        encrypted = True
-
         # Handle geo:-URIs
         if real_text.startswith('geo:'):
             if self.plugin.config['GEO_PREVIEW_PROVIDER'] == 'no_preview':
@@ -271,11 +229,39 @@ class Base(object):
                       'getmap?key={}&center={}&zoom={}&size={},{}&type=map' \
                       '&imagetype=png&pois={},{}&scalebar=false' \
                       .format(apikey, location, zoom, size, size, color,
-                          location)
+                              location)
                 weburl = 'http://www.openstreetmap.org/' \
                          '?mlat={}&mlon={}#map={}/{}/{}&layers=N' \
                          .format(lat, lon, zoom, lat, lon)
                 real_text = url
+        else:
+            weburl = real_text
+            filename = os.path.basename(urlparts.path)
+            ext = os.path.splitext(filename)[1]
+            name = os.path.splitext(filename)[0]
+            namehash = hashlib.sha1(real_text.encode('utf-8')).hexdigest()
+            newfilename = name + '_' + namehash + ext
+            thumbfilename = name + '_' + namehash + '_thumb_' \
+                + str(self.plugin.config['PREVIEW_SIZE']) + ext
+
+            filepath = os.path.join(self.directory, newfilename)
+            thumbpath = os.path.join(self.thumbpath, thumbfilename)
+            filepaths = [filepath, thumbpath]
+
+            key = ''
+            iv = ''
+            encrypted = False
+            if urlparts.fragment:
+                fragment = binascii.unhexlify(urlparts.fragment)
+                key = fragment[16:]
+                iv = fragment[:16]
+                if len(key) == 32 and len(iv) == 16:
+                    encrypted = True
+                if not encrypted:
+                    key = fragment[12:]
+                    iv = fragment[:12]
+                    if len(key) == 32 and len(iv) == 12:
+                        encrypted = True
 
         # file exists but thumbnail got deleted
         if os.path.exists(filepath) and not os.path.exists(thumbpath):
@@ -315,6 +301,37 @@ class Base(object):
                                             repl_end, filepaths, key, iv,
                                             encrypted])
 
+    def _accept_uri(self, urlparts, real_text, additional_data):
+        try:
+            oob_url = additional_data["gajim"]["oob_url"]
+        except (KeyError, AttributeError):
+            oob_url = None
+
+        if not urlparts.netloc:
+            log.info('No netloc found in URL %s', real_text)
+            return False
+
+        # geo
+        if urlparts.scheme == "geo":
+            if self.plugin.config['GEO_PREVIEW_PROVIDER'] == 'no_preview':
+                log.info('geo: link preview is disabled')
+                return False
+            return True
+
+        # aesgcm
+        if urlparts.scheme == 'aesgcm':
+            return True
+
+        # https
+        if urlparts.scheme == 'https':
+            if real_text == oob_url or self.plugin.config['ALLOW_ALL_IMAGES']:
+                return True
+            log.info('Incorrect oob data found')
+            return False
+
+        log.info('Not supported URI scheme found: %s', real_text)
+        return False
+
     def _save_thumbnail(self, thumbpath, mem):
         size = self.plugin.config['PREVIEW_SIZE']
 
@@ -374,11 +391,11 @@ class Base(object):
 
         if image_width > image_height:
             if image_width > size:
-                image_height = int(size / float(image_width) * image_height)
+                image_height = math.ceil((size / float(image_width) * image_height))
                 image_width = int(size)
         else:
             if image_height > size:
-                image_width = int(size / float(image_height) * image_width)
+                image_width = math.ceil((size / float(image_height) * image_width))
                 image_height = int(size)
 
         return image_width, image_height
@@ -476,9 +493,9 @@ class Base(object):
                          key, iv, encrypted):
         file_mime, file_size = tuple_arg
         # Check if mime type is acceptable
-        if (file_mime == '' and file_size == 0) or file_mime is None:
-            log.info("Failed to load HEAD Request for URL: '%s'"
-                     "(see debug log for more info)", url)
+        if not file_mime or not file_size:
+            log.info("Failed to load HEAD Request for URL: '%s' "
+                     "mime: %s, size: %s", url, file_mime, file_size)
             # URL is already displayed
             return
         if file_mime.lower() not in ACCEPTED_MIME_TYPES:
@@ -579,8 +596,12 @@ class Base(object):
         id_ = open_menuitem.connect(
             'activate', self.on_open_menuitem_activate, data)
         self.handlers[id_] = open_menuitem
-        id_ = save_as_menuitem.connect(
-            'activate', self.on_save_as_menuitem_activate, data)
+        if NEW_FILECHOOSER:
+            id_ = save_as_menuitem.connect(
+                'activate', self.on_save_as_menuitem_activate_new, data)
+        else:
+            id_ = save_as_menuitem.connect(
+                'activate', self.on_save_as_menuitem_activate, data)
         self.handlers[id_] = save_as_menuitem
         id_ = copy_link_location_menuitem.connect(
             'activate', self.on_copy_link_location_menuitem_activate, data)
@@ -658,6 +679,25 @@ class Base(object):
         dialog.connect('delete-event', lambda widget, event:
                        on_cancel(widget))
 
+    def on_save_as_menuitem_activate_new(self, menu, data):
+        filepath = data["filepath"]
+        original_filename = data["original_filename"]
+
+        def on_ok(target_path):
+            dirname = os.path.dirname(target_path)
+            if not os.access(dirname, os.W_OK):
+                dialogs.ErrorDialog(
+                    _('Directory "%s" is not writable') % dirname,
+                    _('You do not have permission to '
+                      'create files in this directory.'))
+                return
+            shutil.copy(filepath, target_path)
+
+        FileSaveDialog(on_ok,
+                       path=app.config.get('last_save_dir'),
+                       file_name=original_filename,
+                       transient_for=app.app.get_active_window())
+
     def on_copy_link_location_menuitem_activate(self, menu, data):
         url = data["url"]
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
